@@ -4,10 +4,13 @@ import os
 import re
 import shutil
 from pathlib import Path
+from tempfile import gettempdir, NamedTemporaryFile
 from typing import List, Union, Optional, Tuple
 
 import chmod_monkey
+import requests
 from braceexpand import braceexpand
+
 from ddb.config import config
 from ddb.context import context
 
@@ -95,7 +98,7 @@ def copy_if_different(source, target, read_mode='r', write_mode='w', log=False,
             chmod(target, '+r', logging=False)
 
 
-def force_remove(file: str):
+def force_remove(file: str, silent=False):
     """
     Remove a file, trying to unset readonly flag if a PermissionError occurs.
     """
@@ -105,6 +108,9 @@ def force_remove(file: str):
         # On windows, removing a readonly file raise this error.
         chmod(file, '+w', logging=False)
         os.remove(file)
+    except FileNotFoundError:
+        if not silent:
+            context.log.warning("%s can't be removed because it's already absent", file)
 
 
 def chmod(file: str, mode: str, logging=True):
@@ -121,6 +127,36 @@ def chmod(file: str, mode: str, logging=True):
             context.log.notice("chmod %s %s", mode, file)
 
 
+class FileUtils:
+    """
+    Some file management functions
+    """
+
+    @staticmethod
+    def get_file_content(url: str) -> str:
+        """
+        Get the content of the file
+        :param url: the path to the file (https? or file)
+        :return:
+        """
+        if url.startswith('file://'):
+            return FileUtils._get_local_file_content(url)
+        return requests.get(url).text
+
+    @staticmethod
+    def _get_local_file_content(file_path: str) -> str:
+        """
+        Get the content of the file
+        :param file_path: the path to the file (absolute or relative)
+        :return:
+        """
+        file_path = file_path.replace('file://', '')
+        if not os.path.isabs(file_path):
+            file_path = os.path.join(config.path.project_home, file_path)
+        with open(file_path, 'rb') as file:
+            return file.read().decode("utf-8")
+
+
 class FileWalker:
     """
     Walk files inside project directory.
@@ -165,16 +201,13 @@ class FileWalker:
 
     def _walk(self, *args, recursive=True, **kwargs):
         _walk_generator = os.walk(*args, **kwargs)
-        if not recursive:
-            try:
-                _walk_generator = next(_walk_generator)
-            except StopIteration:
-                return
         for root, dirs, files in os.walk(*args, **kwargs):
             for dirs_item in list(dirs):
-                dirpath = os.path.join(root, dirs_item)
+                dirpath = os.path.relpath(os.path.join(root, dirs_item))
                 if self._is_excluded(dirpath, *self.excludes):
                     dirs.remove(dirs_item)
+                else:
+                    yield dirpath
 
             for files_item in list(files):
                 filepath = os.path.relpath(os.path.join(root, files_item))
@@ -182,6 +215,9 @@ class FileWalker:
                         not self._is_excluded(filepath, *self.excludes):
                     yield filepath
 
+            if not recursive:
+                break
+
     @staticmethod
     def _braceexpand(includes, excludes):
         expanded_includes = []
@@ -318,3 +354,41 @@ class TemplateFinder(FileWalker):
                 return template_candidate[:len(basename) - len(suffix)] + ext, suffix
 
         return None, None
+
+
+def get_single_temporary_file_directory(*args):
+    """
+    Get directory for SingleTemporaryFile *args.
+    """
+    return os.path.join(gettempdir(), *args)
+
+
+class SingleTemporaryFile:
+    """
+    A NamedTemporaryFile wrapper that is created in tmp subdirectory joined with *args strings, using options from
+     **kwargs. It will keep only the last temporary file is subdirectory by cleaning up the directory on each call.
+    """
+
+    def __init__(self, *args, **kwargs):
+        self.tempdir = get_single_temporary_file_directory(*args)
+        os.makedirs(self.tempdir, exist_ok=True)
+
+        if "dir" in kwargs:
+            kwargs.pop("dir")
+        if "delete" in kwargs:
+            kwargs.pop("delete")
+
+        self.named_temporary_file = NamedTemporaryFile(dir=self.tempdir, delete=False, **kwargs)
+
+    def __enter__(self):
+        return self.named_temporary_file.__enter__()
+
+    def __exit__(self, exc_type, exc_value, exc_traceback):
+        tmp_filepath = self.named_temporary_file.name
+        try:
+            return self.named_temporary_file.__exit__(exc_type, exc_value, exc_traceback)
+        finally:
+            for previous_temporary_file in os.listdir(self.tempdir):
+                previous_temporary_filepath = os.path.join(self.tempdir, previous_temporary_file)
+                if previous_temporary_filepath != tmp_filepath:
+                    os.remove(previous_temporary_filepath)
